﻿using System;
using System.Web;
using System.Configuration;

/// <summary>
/// Вспомогательный класс для работы с URL и перенаправления URL 
/// </summary>

public static class UrlTools
{
  // получение домена сайта из конфигурационного файла
  static string siteDomain = ConfigurationManager.AppSettings["SiteDomain"];

  // сохранение ID партнера в виде постоянного cookie-набора
  public static void SaveAffiliate()
  {
    // получение ID партнера
    HttpContext context = HttpContext.Current;
    string affiliateId = context.Request.QueryString["Aff"];

    // партнер?
    if (affiliateId != null)
    {
      // создание нового cookie-набора
      HttpCookie cookie = new HttpCookie("AffiliateID", affiliateId);

      // срок действия cookie - 30 дней
      cookie.Expires = DateTime.Now.AddDays(30);

      // сохранение cookie
      context.Response.Cookies.Add(cookie);
    }
  }

  // проверка, что страница загружается с помощью ее стандартного URL
  // если это не так – перенаправление 301 на стандартный URL
  public static void CheckUrl()
  {
    HttpContext context = HttpContext.Current;
    HttpRequest request = HttpContext.Current.Request;

    // выборка параметров из строки запроса
    string productId = request.QueryString["ProductID"];
    string categoryId = request.QueryString["CategoryID"];

    // исправление URL-адресов категории-продукта
    if (productId != null && categoryId != null)
    {
      CheckCategoryProductUrl(categoryId, productId);
    }
  }

  // проверка URL категории-продукта на согласованность
  // при необходимости перенаправление 301 на нужный URL или возврат кода 404
  public static void CheckCategoryProductUrl(string categoryId, string productId)
  {
    // текущий HttpContext
    HttpContext context = HttpContext.Current;

    // URL, запрошенный посетителем
    string requestedUrl = context.Request.ServerVariables["HTTP_X_REWRITE_URL"];

    // выборка названий продукта и категории из фиктивной базы данных
    string categoryName = SeoData.Categories[categoryId];
    string productName = SeoData.Products[productId];

    // если категории или продукта нет в базе – возврат кода 404
    if (categoryName == null || productName == null)
    {
      Go404();
    }

    // получение стандартной версии URL
    string standardUrl = LinkFactory.MakeCategoryProductUrl
      (categoryName, categoryId, productName, productId);

    // при необходимости перенаправление 301 на нужный URL 
    if (siteDomain + requestedUrl != standardUrl)
    {
      context.Response.Status = "301 Moved Permanently";
      context.Response.AddHeader("Location", standardUrl);
    }
  }

  // Загрузка страницы 404 
  public static void Go404()
  {
    HttpContext.Current.Server.Transfer("~/NotFound.aspx");
  }
}