﻿using System;
using System.Text.RegularExpressions;
using System.Collections.Generic;

/// <summary>
/// Класс для перевода собственной разметки в HTML
/// </summary>

public class CustomMarkup
{
  // хранение регулярных выражений и их строковых замен
  private static Dictionary<Regex, string> rs;

  // трансляция собственной разметки из входной строки в HTML
  public static string Translate(string markupString)
  {
    // применение каждого регулярного выражения из коллекции
    foreach (Regex x in rs.Keys)
    {
      markupString = x.Replace(markupString, rs[x]);
    }

    // возврат преобразованной строки
    return markupString;
  }

  // статический конструктор для создания коллекции 
  // объектов Regex и заменяющих строк для трансляции
  static CustomMarkup()
  {
    // создание коллекции
    rs = new Dictionary<Regex, string>();

    // трансляция дескрипторов жирного шрифта
    rs.Add(new Regex(@"\{bold}(.*?)\{/bold}", 
      RegexOptions.IgnoreCase), @"<b>$1</b>");

    // трансляция дескрипторов курсива
    rs.Add(new Regex(@"\{italic}(.*?)\{/italic}", 
      RegexOptions.IgnoreCase), @"<i>$1</i>");

    // трансляция дескрипторов подчеркивания
    rs.Add(new Regex(@"\{underline}(.*?)\{/underline}", 
      RegexOptions.IgnoreCase), @"<u>$1</u>");

    // трансляция заголовков h1 
    rs.Add(new Regex(@"\{heading}(.*?)\{/heading}", 
      RegexOptions.IgnoreCase), @"<h1 class=some_class>$1</h1>");

    // трансляция заголовков h2 
    rs.Add(new Regex(@"\{subheading}(.*?)\{/subheading}", 
      RegexOptions.IgnoreCase), @"<h2 class=some_other_class>$1</h2>");

    // преобразование гиперссылок
    rs.Add(new Regex(@"\{link:(.*?)}(.*?)\{/link}", 
      RegexOptions.IgnoreCase), @"<a href=\’$1\’>$2</a>");

    // преобразование гиперссылок, открытых в новом окне
    rs.Add(new Regex(@"\{elink:(.*?)}(.*?)\{/elink}", 
      RegexOptions.IgnoreCase), @"<a href=\'$1\' target=""_blank"">$2</a>");

    // трансляция дескрипторов неупорядоченных списков
    rs.Add(new Regex(@"\{unordered-list}(.*?)\{/unordered-list}\s*",
      RegexOptions.IgnoreCase), @"<ul>$1</ul>");

    // трансляция дескрипторов неупорядоченных списков
    rs.Add(new Regex(@"\{ordered-list}(.*?)\{/ordered-list}\s*", 
      RegexOptions.IgnoreCase), @"<ol>$1</ol>");

    // преобразование элементов списков
    rs.Add(new Regex(@"\\s*{list-element}(.*?)\{/list-element}\s*",
      RegexOptions.IgnoreCase), @"<li>$1</li>");

    // трансляция дескрипторов изображений
    rs.Add(new Regex(@"\{picture:(.*?)} ",
      RegexOptions.IgnoreCase), @"<img src=""$1"" border=""0"">");

    // удаление символов табуляции
    rs.Add(new Regex(@"\t"), "");

    // удаление комментариев
    rs.Add(new Regex(@"\{comment}(.*?)\{/comment}",
      RegexOptions.IgnoreCase), "");
  }
}