﻿using System;
using System.Collections;
using System.Collections.Generic;

/// <summary>
/// Коллекция объектов Social 
/// </summary>

public class SocialCollection : IEnumerable
{
  // внутренняя коллекция объектов Social хранится в виде словаря
  private Dictionary<string, Social> socials;

  // инициализация словаря в конструкторе
  public SocialCollection()
  {
    socials = new Dictionary<string, Social>();
  }

  // индексатор для выборки элементов по имени
  public Social this[string name]
  {
    get
    {
      // ключи содержат только строчные буквы
      name = name.ToLower();

      // возврат объекта Social
      if (socials.ContainsKey(name))
        return socials[name];
      else
        throw new ArgumentException("Неизвестый социальный сайт: " + name);
    }
  }

  // создание нового объекта Social и занесение его в коллекцию
  public void Add(string name, string imageFile, string url)
  {
    // замена прописных букв в ключе на прописные
    name = name.ToLower();

    // занесение объекта Social в коллекцию
    if (!socials.ContainsKey(name))
    {
      // создание нового объекта Social 
      Social social = new Social(name, imageFile, url);

      // добавление нового объекта в коллекцию
      socials.Add(name, social);
    }
    else
      throw new ArgumentException("Объект уже существует: " + name);
  }

  // добавление объекта Social в коллекцию
  public void Add(Social social)
  {
    // добавление объекта Social в коллекцию
    if (!socials.ContainsKey(social.Name.ToLower()))
      socials.Add(social.Name.ToLower(), social);
    else
      throw new ArgumentException("Объект уже существует: " + social.Name);
  }

  // код инфраструктуры, реализующий IEnumerable
  IEnumerator IEnumerable.GetEnumerator()
  {
    return GetEnumerator();
  }

  public IEnumerator<Social> GetEnumerator()
  {
    foreach (Social item in socials.Values)
    {
      yield return item;
    }
  }
}