﻿using System;
using System.Text.RegularExpressions;

/// <summary>
/// NoFollow предназначен для добавления rel=nofollow к подозрительным ссылкам
/// </summary>

public static class NoFollow
{
  // белый список доменов (строчными буквами)
  private static string[] whitelist = { "seoasp", "www.seoegghead.com", "www.cristiandarie.ro" };

  // поиск всех ссылок во входной строке и их обраотка с помощью fixLink
  public static string FixLinks(string input)
  {
    // определение компонента сравнения
    MatchEvaluator fixThisLink = new MatchEvaluator(NoFollow.fixLink);

    // изменение ссылок во входной строке
    string fixedInput = Regex.Replace(input,
                                      "(<a.*?>)",
                                      fixThisLink,
                                      RegexOptions.IgnoreCase);

    // возврат измененной входной строки
    return fixedInput;
  }

  // получение результата сравнения Regex со ссылкой наподобие 
  // <a href="http://too.much.spam/"> и при необходимости добавление ref=nofollow 
  private static string fixLink(Match linkMatch)
  {
    // выделение ссылки из полученного Match
    string singleLink = linkMatch.Value;

    // если в ссылке уже есть rel=nofollow, возврат без изменения
    if (Regex.IsMatch(singleLink,
                      @"rel\s*?=\s*?['""]?.*?nofollow.*?['""]?",
                      RegexOptions.IgnoreCase))
    {
      return singleLink;
    }

    // извлечение URL из ссылки с помощью выделенной группы
    Match m = Regex.Match(singleLink,
                          @"href\s*?=\s*?['""]?(?<url>[^'""]*)['""]?",
                          RegexOptions.IgnoreCase);
    string url = m.Groups["url"].Value;

    // если в URL нет http://, считаем, что это локальная ссылка
    if (!url.Contains("http://"))
    {
      return singleLink;
    }

    // извлечение имени хоста (наподобие www.cristiandarie.ro) из URL
    Uri uri = new Uri(url);
    string host = uri.Host.ToLower();

    // если хост присутствует в белом списке, не изменяем его
    if (Array.IndexOf(whitelist, host) >= 0)
    {
      return singleLink;
    }

    // если в URL уже есть атрибут rel, изменяем его значение на nofollow
    string newLink = Regex.Replace(singleLink,
             @"(?<a>rel\s*=\s*(?<b>['""]?))((?<c>[^'""\s]*|[^'""]*))(?<d>['""]?)?",
             "${a}nofollow${d}",
             RegexOptions.IgnoreCase);

    // если в строке есть измененный нами атрибут rel, возвращаем новую ссылку
    if (newLink != singleLink)
    {
      return newLink;
    }

    // если дошли до этого места, то в ссылку нужно добавить rel=nofollow 
    newLink = Regex.Replace(singleLink, "<a", @"<a rel=""nofollow""",
                            RegexOptions.IgnoreCase);
    return newLink;
  }
}