﻿using System;
using System.Text.RegularExpressions;

/// <summary>
/// Sanitize предназначен для удаления недопустимых дескрипторов или атрибутов
/// </summary>

public static class Sanitize
{
  // список допустимых или безвредных дескрипторов (строчными буквами)
  private static string[] allowedTags = { "p", "h1", "b", "i", "a", "ul", "li", "pre", "hr", "blockquote", "img" };

  // список атрибутов, требующих зачистки
  private static string badAttributes =
    "onerror|onmousemove|onmouseout|onmouseover|" +
    "onkeypress|onkeydown|onkeyup|javascript:";

  // зачистка HTML-кода в $inputHTML
  public static string FixTags(string inputHtml)
  {
    // определение компонента сравнения
    MatchEvaluator fixThisLink = new MatchEvaluator(Sanitize.fixTag);

    // обработка всех дескрипторов из входной строки по порядку
    string fixedHtml = Regex.Replace(inputHtml,
                                     "(<.*?>)",
                                     fixThisLink,
                                     RegexOptions.IgnoreCase);

    // возврат измененной входной строки
    return fixedHtml;
  }

  // удаление дескриптора, если его нет в списке разрешенных
  private static string fixTag(Match tagMatch)
  {
    string tag = tagMatch.Value;

    // выборка имени дескриптора, наподобие "a" or "h1"”
    Match m = Regex.Match(tag,
                          @"</?(?<tagName>[^\s/]*)[>\s/]",
                          RegexOptions.IgnoreCase);
    string tagName = m.Groups["tagName"].Value.ToLower();

    // если дескриптора нет в списке разрешенных — удаляем
    if (Array.IndexOf(allowedTags, tagName) < 0)
    {
      return "";
    }

    // удаление из дескриптора опасных атрибутов
    string fixedTag = Regex.Replace(tag,
                                    "(" + Sanitize.badAttributes + @")(\s*)(?==)",
                                    "SANITIZED", RegexOptions.IgnoreCase);

    // возвращение измененного дескриптора
    return fixedTag;
  }
}