﻿using System;
using System.Web;
using System.Configuration;
using System.Text.RegularExpressions;

/// <summary>
/// LinkFactory создает абсолютные ссылки на местоположения в сайте
/// </summary>

public static class LinkFactory
{
  // регулярное выражение для удаления символов, не совпадающих с
  // a-z, 0-9, дефисом, подчерком или пробелом
  private static Regex purifyUrlRegex =
    new Regex("[^-a-zA-Z0-9_ ]", RegexOptions.Compiled);

  // регулярное выражение, заменяющее дефисы, подчерки и пробелы на дефисы
  private static Regex dashesRegex =
    new Regex("[-_ ]+", RegexOptions.Compiled);

  // создание ссылки на страницу продукта
  public static string MakeCategoryProductUrl(
    string categoryName, string categoryId, string productName, string productId)
  {
    // подготовка названия продукта и названия категории для включения в URL
    categoryName = prepareUrlText(categoryName);
    productName = prepareUrlText(productName);

    // чтение домена сайта из конфигурационного файла
    string siteDomain = ConfigurationManager.AppSettings["SiteDomain"];

    // создание URL с ключевыми словами
    string url = String.Format("{0}/Products/{1}-C{2}/{3}-P{4}.html",
      siteDomain, categoryName, categoryId, productName, productId);

    // возврат URL
    return HttpUtility.UrlPathEncode(url);
  }

  // создание ссылки на медиафайл
  public static string MakeMediaUrl(string name, string extension, string id)
  {
    // подготовка названия носителя для включеня в URL
    name = prepareUrlText(name);

    // чтение домена сайта из конфигурационного файла
    string siteDomain = ConfigurationManager.AppSettings["SiteDomain"];

    // создание URL с ключевыми словами 
    string url = String.Format("{0}/{1}-M{2}.{3}", siteDomain, name, id, extension);

    // возврат URL
    return HttpUtility.UrlPathEncode(url);
  }

  // подготовка строки для включения в URL
  private static string prepareUrlText(string urlText)
  {
    // удаление символов, не совпадающих с
    // a-z, 0-9, дефисом, подчерком или пробелом
    urlText = purifyUrlRegex.Replace(urlText, "");

    // удаление пробелов из начала и конца строки
    urlText = urlText.Trim();

    // замена всех дефисов, подчерков и пробелов на дефисы
    urlText = dashesRegex.Replace(urlText, "-");

    // возврат измененной строки
    return urlText;
  }
}