﻿<%@ WebHandler Language="C#" Class="Sitemap" %>

using System;
using System.Web;
using System.Xml;
using System.Configuration;
using System.Collections.Generic;

public class Sitemap : IHttpHandler
{
  // структура для представления элемента карты сайта
  struct SitemapItem
  {
    // общедоступные поля свойств элемента карты сайта
    public string Url, LastMod, ChangeFreq, Priority;

    // конструктор, инициализирующий поля
    public SitemapItem(string url,
      string lastMod, string changeFreq, string priority)
    {
      this.Url = url;
      this.LastMod = lastMod;
      this.ChangeFreq = changeFreq;
      this.Priority = priority;
    }
  }

  // точка входа – метод, вызываемый при загрузке обработчика
  public void ProcessRequest(HttpContext context)
  {
    // получение значения параметра Target 
    string target = context.Request.QueryString["Target"];

    // если такого параметра нет - выход
    if (target == null) return;

    // генерация запрошенной карты сайта
    if (target.ToLower() == "google")
    {
      GenerateGoogleSitemap();
    }
    else if (target.ToLower() == "yahoo")
    {
      GenerateYahooSitemap();
    }
  }

  // генерация карты сайта для Google 
  private void GenerateGoogleSitemap()
  {
    // получение текущего объекта HttpResponse 
    HttpResponse response = HttpContext.Current.Response;

    // указание типа контента
    response.ContentType = "text/xml";

    // генерация карты сайта Google с помощью XmlWriter 
    XmlWriter xmlWriter = XmlWriter.Create(response.OutputStream);

    // запись первого элемента
    xmlWriter.WriteStartElement("urlset", 
      "http://www.google.com/schemas/sitemap/0.84");

    // получение списка элементов карты
    List<SitemapItem> sitemapItems = GetSitemapItems();

    // генерация элементов карты
    foreach (SitemapItem sitemapItem in sitemapItems)
    {
      // генерация элемента <url> и его содержимого
      xmlWriter.WriteStartElement("url");
      xmlWriter.WriteElementString("loc", sitemapItem.Url);
      xmlWriter.WriteElementString("lastmod", sitemapItem.LastMod);
      xmlWriter.WriteElementString("changefreq", sitemapItem.ChangeFreq);
      xmlWriter.WriteElementString("priority", sitemapItem.Priority);
      xmlWriter.WriteEndElement();
    }

    // закрытие документа
    xmlWriter.WriteEndElement();    
    xmlWriter.Flush();
  }

  // генерация карты сайта для Yahoo 
  private void GenerateYahooSitemap()
  {
    // получение текущего объекта HttpResponse 
    HttpResponse response = HttpContext.Current.Response;

    // указание типа контента
    response.ContentType = "text/plain";

    // получение списка элементов карты
    List<SitemapItem> sitemapItems = GetSitemapItems();

    // генерация элементов карты
    foreach (SitemapItem sitemapItem in sitemapItems)
    {
      response.Write(sitemapItem.Url + "\n");
    }
  }

  // создание списка элементов, которые нужно включить в карту сайта
  private List<SitemapItem> GetSitemapItems()
  {
    // объявление списка элементов карты
    List<SitemapItem> sitemapItems = new List<SitemapItem>();

    // создание списка URL-адресов для включения в карту
    sitemapItems.Add(new SitemapItem(
      ConfigurationManager.AppSettings["SiteDomain"] + "/Catalog.html",
      "2007/05/05", "daily", "0.3"));

    sitemapItems.Add(new SitemapItem(
      LinkFactory.MakeCategoryProductUrl(SeoData.Categories["12"], "12",
      SeoData.Products["45"], "45"), "2007/05/05", "weekly", "0.6"));

    sitemapItems.Add(new SitemapItem(
      LinkFactory.MakeCategoryProductUrl(SeoData.Categories["2"], "2",
      SeoData.Products["42"], "42"), "2007/05/05", "weekly", "0.6"));

    sitemapItems.Add(new SitemapItem(
      LinkFactory.MakeCategoryProductUrl(SeoData.Categories["6"], "6",
      SeoData.Products["15"], "15"), "2007/05/05", "weekly", "0.6"));

    sitemapItems.Add(new SitemapItem(
      LinkFactory.MakeCategoryProductUrl(SeoData.Categories["6"], "6",
      SeoData.Products["31"], "31"), "2007/05/05", "weekly", "0.6"));

    // возврат списка объектов Sitemap 
    return sitemapItems;
  }

  public bool IsReusable
  {
    get
    {
      return false;
    }
  }
}