﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;

/// <summary>
/// Class с интерфейсом к базе данных CloakData 
/// </summary>

public static class CloakDb
{
  // возвращает дату и версию последнего обновления
  public static bool GetLastUpdate(ref string version, ref DateTime updatedOn)
  {
    // создание объекта SqlCommand с подключением к базе данных маскирования
    SqlCommand command = CloakDb.createCommand("CloakDatabase");

    // указание имени хранимой процедуры, которую требуется выполнить
    command.CommandText = "CloakGetLastUpdate";

    // создание нового объекта DataTable
    DataTable results = new DataTable();

    // выполнение хранимой процедуры и загрузка результатов в DataTable
    command.Connection.Open();
    try { results.Load(command.ExecuteReader()); }
    finally { command.Connection.Close(); }

    // возврат false, если запись для обновления не найдена
    if (results.Rows.Count == 0)
    {
      return false;
    }
    else
    {
      version = results.Rows[0]["Version"].ToString();
      updatedOn = (DateTime)results.Rows[0]["UpdatedOn"];
      return true;
    }
  }

  // сохранение времени и даты последнего обновления базы данных
  public static void SetLastUpdate(string version, DateTime updatedOn)
  {
    // создание объекта SqlCommand с подключением к базе данных маскирования
    SqlCommand command = CloakDb.createCommand("CloakDatabase");

    // указание имени хранимой процедуры, которую требуется выполнить
    command.CommandText = "CloakSetLastUpdate";

    // добавление параметра хранимой процедуры
    SqlParameter p = new SqlParameter("@NewVersion", version);
    command.Parameters.Add(p);

    // добавление параметра хранимой процедуры
    p = new SqlParameter("@UpdatedOn", updatedOn);
    command.Parameters.Add(p);

    // выполнение хранимой процедуры и загрузка результатов в DataTable
    command.Connection.Open();
    try { command.ExecuteNonQuery(); }
    finally { command.Connection.Close(); }
  }

  // сохранение времени и даты последнего обновления базы данных
  public static void AddSpiderRecord(string spiderName, string recordType, string value)
  {
    // создание объекта SqlCommand с подключением к базе данных маскирования
    SqlCommand command = CloakDb.createCommand("CloakDatabase");

    // указание имени хранимой процедуры, которую требуется выполнить
    command.CommandText = "CloakAddSpiderRecord";

    // @SpiderName
    SqlParameter p = new SqlParameter("@SpiderName", spiderName);
    command.Parameters.Add(p);

    // @RecordType
    p = new SqlParameter("@RecordType", recordType);
    command.Parameters.Add(p);

    // добавление параметра хранимой процедуры
    p = new SqlParameter("@Value", value);
    command.Parameters.Add(p);

    // выполнение хранимой процедуры и загрузка результатов в DataTable
    command.Connection.Open();
    try { command.ExecuteNonQuery(); }
    finally { command.Connection.Close(); }
  }

  // удаление из базы данных записей о поисковом агенте
  public static void DeleteSpider(string spiderName)
  {
    // создание объекта SqlCommand с подключением к базе данных маскирования
    SqlCommand command = CloakDb.createCommand("CloakDatabase");

    // указание имени хранимой процедуры, которую требуется выполнить
    command.CommandText = "CloakDeleteSpider";

    // @SpiderName
    SqlParameter p = new SqlParameter("@SpiderName", spiderName);
    command.Parameters.Add(p);

    // выполнение хранимой процедуры и загрузка результатов в DataTable
    command.Connection.Open();
    try { command.ExecuteNonQuery(); }
    finally { command.Connection.Close(); }
  }

  // проверка, является ли данный пользовательский агент указанным пауком
  // (если строка spiderName пуста, проверяются все пауки)
  public static bool CheckUA(string uaValue, string spiderName)
  {
    // создание объекта SqlCommand с подключением к базе данных маскирования
    SqlCommand command = CloakDb.createCommand("CloakDatabase");

    // указание имени хранимой процедуры, которую требуется выполнить
    command.CommandText = "CloakCheckUA";

    // @UAValue
    SqlParameter p = new SqlParameter("@UAValue", uaValue);
    command.Parameters.Add(p);

    // @SpiderName
    p = new SqlParameter("@SpiderName", spiderName);
    command.Parameters.Add(p);

    // выполнение хранимой процедуры
    command.Connection.Open();
    int numberOfMatches;
    try { numberOfMatches = (int)command.ExecuteScalar(); }
    finally { command.Connection.Close(); }

    // возврат true при обнаружении соответствия
    return (numberOfMatches > 0);
  }

  // проверка, является ли данный IP указанным пауком
  // (если строка spiderName пуста, проверяются все пауки)
  public static bool CheckIP(string ipValue, string spiderName)
  {
    // создание объекта SqlCommand с подключением к базе данных маскирования
    SqlCommand command = CloakDb.createCommand("CloakDatabase");

    // указание имени хранимой процедуры, которую требуется выполнить
    command.CommandText = "CloakCheckIP";

    // @UAValue
    SqlParameter p = new SqlParameter("@IPValue", ipValue);
    command.Parameters.Add(p);

    // @SpiderName
    p = new SqlParameter("@SpiderName", spiderName);
    command.Parameters.Add(p);

    // выполнение хранимой процедуры
    command.Connection.Open();
    int numberOfMatches;
    try { numberOfMatches = (int)command.ExecuteScalar(); }
    finally { command.Connection.Close(); }

    // возврат true при обнаружении соответствия
    return (numberOfMatches > 0);
  }

  // создание и подготовка нового объекта SqlCommand для нового подключения
  private static SqlCommand createCommand(string connectionName)
  {
    // получение строки подключения к базе данных
    string connectionString =
      ConfigurationManager.ConnectionStrings[connectionName].ConnectionString;

    // получение подключения к конкретной базе данных
    SqlConnection conn = new SqlConnection(connectionString);

    // создание объекта команды для конкретной базы данных
    SqlCommand comm = conn.CreateCommand();

    // указание типа команды хранимой процедуре
    comm.CommandType = CommandType.StoredProcedure;

    // возврат инициализированного объекта команды
    return comm;
  }
}