﻿using System;
using System.Web;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;

/// <summary>
/// Краткое описание CatalogDb
/// </summary>

public class CatalogDb
{
  // выборка информации о брэнде
  public static DataRow GetBrand(string brandId)
  {
    // создание объекта SqlCommand с подключением к базе данных продуктов
    SqlCommand command = createCommand("ProductsDatabase");

    // указание имени хранимой процедуры, которую нужно выполнить
    command.CommandText = "GetBrand";

    // @BrandID
    SqlParameter p = new SqlParameter("@BrandID", brandId);
    command.Parameters.Add(p);

    // создание нового объекта DataTable
    DataTable results = new DataTable();

    // выполнение хранимой процедуры и загрузка результатов в DataTable
    command.Connection.Open();
    try { results.Load(command.ExecuteReader()); }
    finally { command.Connection.Close(); }

    // возврат DataRow с информацией о категории
    return (results.Rows.Count > 0) ? results.Rows[0] : null;
  }

  // выборка всех категорий
  public static DataTable GetCategories()
  {
    // создание объекта SqlCommand с подключением к базе данных продуктов
    SqlCommand command = createCommand("ProductsDatabase");

    // указание имени хранимой процедуры, которую нужно выполнить
    command.CommandText = "GetCategories";

    // создание нового объекта DataTable
    DataTable results = new DataTable();

    // выполнение хранимой процедуры и загрузка результатов в DataTable
    command.Connection.Open();
    try { results.Load(command.ExecuteReader()); }
    finally { command.Connection.Close(); }

    // возврат результатов
    return results;
  }

  // выборка информации о категории
  public static DataRow GetCategory(string categoryId)
  {
    // создание объекта SqlCommand с подключением к базе данных продуктов
    SqlCommand command = createCommand("ProductsDatabase");

    // указание имени хранимой процедуры, которую нужно выполнить
    command.CommandText = "GetCategory";

    // @CategoryID
    SqlParameter p = new SqlParameter("@CategoryID", categoryId);
    command.Parameters.Add(p);

    // содание нового объекта DataTable
    DataTable results = new DataTable();

    // выполнение хранимой процедуры и загрузка результатов в DataTable
    command.Connection.Open();
    try { results.Load(command.ExecuteReader()); }
    finally { command.Connection.Close(); }

    // возврат DataRow с информацией о категории
    return (results.Rows.Count > 0) ? results.Rows[0] : null;
  }

  // выборка страницы продуктов из категории
  public static DataTable GetProductsInCategory(string categoryId, int pageNumber, out int howManyPages)
  {
    // создание объекта SqlCommand с подключением к базе данных продуктов
    SqlCommand command = createCommand("ProductsDatabase");

    // указание имени хранимой процедуры, которую нужно выполнить
    command.CommandText = "GetProductsInCategory";

    // @CategoryID
    SqlParameter p = new SqlParameter("@CategoryID", categoryId);command.Parameters.Add(p);

    // @PageNumber
    p = new SqlParameter("@PageNumber", pageNumber);
    command.Parameters.Add(p);

    // @PageNumber
    p = new SqlParameter("@ProductsPerPage",
      ConfigurationManager.AppSettings["ProductsPerPage"]);
    command.Parameters.Add(p);

    // @HowManyPages
    p = new SqlParameter("@HowManyProducts", SqlDbType.Int);
    p.Direction = ParameterDirection.Output;
    command.Parameters.Add(p);

    // создание нового объекта DataTable
    DataTable results = new DataTable();

    // выполнение хранимой процедуры и загрузка результатов в DataTable
    command.Connection.Open();
    try { results.Load(command.ExecuteReader()); }
    finally { command.Connection.Close(); }

    // возврат количества страниц продуктов в выходном параметре
    int howManyProducts =
      Int32.Parse(command.Parameters["@HowManyProducts"].Value.ToString());
    howManyPages = (int)Math.Ceiling((double)howManyProducts /
      double.Parse(ConfigurationManager.AppSettings["ProductsPerPage"]));

    // возврат результатов
    return results;
  }

  // выборка информации об одном продукте
  public static DataRow GetProduct(string productId)
  {
    // создание объекта SqlCommand с подключением к базе данных продуктов
    SqlCommand command = createCommand("ProductsDatabase");

    // указание имени хранимой процедуры, которую нужно выполнить
    command.CommandText = "GetProduct";

    // @CategoryID
    SqlParameter p = new SqlParameter("@ProductID", productId);
    command.Parameters.Add(p);

    // создание нового объекта DataTable
    DataTable results = new DataTable();

    // выполнение хранимой процедуры и загрузка результатов в DataTable
    command.Connection.Open();
    try { results.Load(command.ExecuteReader()); }
    finally { command.Connection.Close(); }

    // возврат DataRow с информацией о продукте
    return (results.Rows.Count > 0) ? results.Rows[0] : null;
  }

  // выборка продуктов со вторичными категориями 
  // для исключения с помощью файла robots.txt
  public static DataTable GetSecondaryProductCategories()
  {
    // создание объекта SqlCommand с подключением к базе данных продуктов
    SqlCommand command = createCommand("ProductsDatabase");

    // указание имени хранимой процедуры, которую нужно выполнить
    command.CommandText = "GetSecondaryProductCategories";

    // создание нового объекта DataTable
    DataTable results = new DataTable();

    // выполнение хранимой процедуры и загрузка результатов в DataTable
    command.Connection.Open();
    try { results.Load(command.ExecuteReader()); }
    finally { command.Connection.Close(); }

    // возврат результатов
    return results;
  }

  // создание и подготовка нового объекта SqlCommand для нового подключения
  private static SqlCommand createCommand(string connectionName)
  {
    // получение строки подключения к базе данных
    string connectionString =
      ConfigurationManager.ConnectionStrings[connectionName].ConnectionString;

    // получение объекта подключения для конкретной базы данных
    SqlConnection conn = new SqlConnection(connectionString);

    // создание объекта команды для конкретной базы данных
    SqlCommand comm = conn.CreateCommand();

    // указание типа команды хранимой процедуре
    comm.CommandType = CommandType.StoredProcedure;

    // возврат подготовленного объекта команды
    return comm;
  }
}