﻿using System;
using System.Web;
using System.Configuration;
using System.Text.RegularExpressions;

/// <summary>
/// LinkFactory создает абсолютные ссылки на местоположения в нашем сайте
/// </summary>

public static class LinkFactory
{
  // регулярное выражение для удаления символов, не совпадающих с
  // a-z, 0-9, дефисами, подчерками и пробелами
  private static Regex purifyUrlRegex = new Regex("[^-a-zA-Z0-9_ ]", RegexOptions.Compiled);

  // регулярное выражение для замены дефисов, подчерков и пробелов на дефисы
  private static Regex dashesRegex = new Regex("[-_ ]+", RegexOptions.Compiled);

  // создание ссылки на первую страницу продуктов в категории
  public static string MakeCategoryUrl(string categoryName, string categoryId)
  {
    return MakeCategoryUrl(categoryName, categoryId, 1);
  }

  // создание ссылки на подстраницу категории
  public static string MakeCategoryUrl(string categoryName, string categoryId, int page)
  {
    // подготовка названия категории для включения в URL
    categoryName = prepareUrlText(categoryName);

    // чтение домена сайта из конфигурационного файла
    string siteDomain = ConfigurationManager.AppSettings["SiteDomain"];

    // создание URL с ключевыми словами
    string url = String.Format("{0}/Products/{1}-C{2}/",
      siteDomain, categoryName, categoryId);
    if (page != 1)
      url += String.Format("Page-{0}/", page);

    // возврат URL
    return HttpUtility.UrlPathEncode(url);
  }

  // создание ссылки на страницу продукта
  public static string MakeCategoryProductUrl(
    string categoryName, string categoryId, string productName, string productId)
  {
    // подготовка названия продукта и названия категории для включения в URL
    categoryName = prepareUrlText(categoryName);
    productName = prepareUrlText(productName);

    // чтение домена сайта из конфигурационного файла
    string siteDomain = ConfigurationManager.AppSettings["SiteDomain"];

    // создание URL с ключевыми словами
    string url = String.Format("{0}/Products/{1}-C{2}/{3}-P{4}.html",
      siteDomain, categoryName, categoryId, productName, productId);

    // возврат URL
    return HttpUtility.UrlPathEncode(url);
  }

  // создание ссылки на медиафайл
  public static string MakeMediaUrl(string name, string extension, string id)
  {
    // подготовка имени носителя для включения в URL
    name = prepareUrlText(name);

    // чтение домена сайта из конфигурационного файла
    string siteDomain = ConfigurationManager.AppSettings["SiteDomain"];

    // создание URL с ключевыми словами
    string url = String.Format("{0}/{1}-M{2}.{3}", siteDomain, name, id, extension);

    // возврат URL
    return HttpUtility.UrlPathEncode(url);
  }

  // подготовка строки для включения в URL
  private static string prepareUrlText(string urlText)
  {
    // удаление всех символов, не совпадающих с
    // a-z, 0-9, дефисами, подчерками и пробелами
    urlText = purifyUrlRegex.Replace(urlText, "");

    // удаление всех начальных и завершающих проблеов
    urlText = urlText.Trim();

    // замена всех дефисов, подчерков и пробелов на дефисы
    urlText = dashesRegex.Replace(urlText, "-");

    // возврат модифицированной строки
    return urlText;
  }
}